package cn.com.qmth.examcloud.starters.face.verify.service.impl;

import cn.com.qmth.examcloud.starters.face.verify.FaceVerifyProperties;
import cn.com.qmth.examcloud.starters.face.verify.common.CommonUtils;
import cn.com.qmth.examcloud.starters.face.verify.common.JsonHelper;
import cn.com.qmth.examcloud.starters.face.verify.model.FaceResult;
import cn.com.qmth.examcloud.starters.face.verify.model.baidu.BaiduApiHelper;
import cn.com.qmth.examcloud.starters.face.verify.model.faceplus.FacePlusApiHelper;
import cn.com.qmth.examcloud.starters.face.verify.model.param.FaceApiParam;
import cn.com.qmth.examcloud.starters.face.verify.model.param.FaceApiType;
import cn.com.qmth.examcloud.starters.face.verify.model.param.ImageParm;
import cn.com.qmth.examcloud.starters.face.verify.model.param.ImageParmType;
import cn.com.qmth.examcloud.starters.face.verify.service.FaceVerifyService;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FaceVerifyServiceImpl implements FaceVerifyService {

    private final static Logger log = LoggerFactory.getLogger(FaceVerifyServiceImpl.class);

    private FaceVerifyProperties properties;

    /**
     * 人脸真实性检测
     */
    @Override
    public FaceResult faceVerify(FaceApiParam apiParam) {
        if (ArrayUtils.getLength(apiParam.getImages()) < 1) {
            throw new IllegalArgumentException("至少传一个图片参数！");
        }

        if (FaceApiType.FACE_PLUS == apiParam.getApiType()) {
            throw new IllegalArgumentException("FacePlus暂不支持人脸真实性检测！");
        }

        Map<String, String> imageData = this.buildImageParmForBaidu(apiParam.getImages()[0]);
        imageData.put("face_field", "spoofing");
        List<Map<String, String>> images = new ArrayList<>();
        images.add(imageData);
        String params = new JsonHelper().toJson(images);

        if (FaceApiType.BAIDU_API == apiParam.getApiType()) {
            return BaiduApiHelper.faceVerify(properties, params);
        }
        // 默认：百度私有化部署API
        return BaiduApiHelper.faceVerifyUseLocalApi(properties, params);
    }

    /**
     * 人脸识别
     */
    @Override
    public FaceResult faceDetect(FaceApiParam apiParam) {
        if (ArrayUtils.getLength(apiParam.getImages()) < 1) {
            throw new IllegalArgumentException("至少传一个图片参数！");
        }

        if (FaceApiType.FACE_PLUS == apiParam.getApiType()) {
            Map<String, String> params = new HashMap<>();
            this.buildImageParmForFacePlus(params, apiParam.getImages()[0], null);

            return FacePlusApiHelper.faceDetect(properties, params);
        }

        Map<String, String> imageData = this.buildImageParmForBaidu(apiParam.getImages()[0]);
        imageData.put("liveness_control", "NORMAL");
        imageData.put("max_face_num", "2");
        String params = new JsonHelper().toJson(imageData);

        if (FaceApiType.BAIDU_API == apiParam.getApiType()) {
            return BaiduApiHelper.faceDetect(properties, params);
        }
        // 默认：百度私有化部署API
        return BaiduApiHelper.faceDetectUseLocalApi(properties, params);
    }

    /**
     * 人脸比对
     */
    @Override
    public FaceResult faceCompare(FaceApiParam apiParam) {
        if (ArrayUtils.getLength(apiParam.getImages()) < 2) {
            throw new IllegalArgumentException("至少传两个图片参数！");
        }

        if (FaceApiType.FACE_PLUS == apiParam.getApiType()) {
            Map<String, String> params = new HashMap<>();
            this.buildImageParmForFacePlus(params, apiParam.getImages()[0], "1");
            this.buildImageParmForFacePlus(params, apiParam.getImages()[1], "2");

            return FacePlusApiHelper.faceCompare(properties, params);
        }

        List<Map<String, String>> images = new ArrayList<>();
        images.add(this.buildImageParmForBaidu(apiParam.getImages()[0]));
        images.add(this.buildImageParmForBaidu(apiParam.getImages()[1]));
        String params = new JsonHelper().toJson(images);

        Double expectFaceCompareScore;
        if (apiParam.getExpectFaceCompareScore() != null) {
            expectFaceCompareScore = apiParam.getExpectFaceCompareScore();
        } else {
            expectFaceCompareScore = properties.getBaiduExpectFaceCompareScore();
        }

        if (FaceApiType.BAIDU_API == apiParam.getApiType()) {
            return BaiduApiHelper.faceCompare(properties, params, expectFaceCompareScore);
        }
        // 默认：百度私有化部署API
        return BaiduApiHelper.faceCompareUseLocalApi(properties, params, expectFaceCompareScore);
    }

    @Override
    public FaceResult faceVerifyByBaidu(ImageParm image) {
        Map<String, String> imageData = this.buildImageParmForBaidu(image);
        imageData.put("face_field", "spoofing");

        List<Map<String, String>> images = new ArrayList<>();
        images.add(imageData);

        String params = new JsonHelper().toJson(images);
        if (properties.getBaiduLocalEnabled()) {
            return BaiduApiHelper.faceVerifyUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceVerify(properties, params);
    }

    @Override
    public FaceResult faceVerifyByBaidu(ImageParm image, boolean localEnabled) {
        Map<String, String> imageData = this.buildImageParmForBaidu(image);
        imageData.put("face_field", "spoofing");

        List<Map<String, String>> images = new ArrayList<>();
        images.add(imageData);

        String params = new JsonHelper().toJson(images);
        if (localEnabled) {
            return BaiduApiHelper.faceVerifyUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceVerify(properties, params);
    }

    @Override
    public FaceResult faceDetectByBaidu(ImageParm image) {
        Map<String, String> imageData = this.buildImageParmForBaidu(image);
        imageData.put("liveness_control", "NORMAL");
        imageData.put("max_face_num", "2");

        String params = new JsonHelper().toJson(imageData);
        if (properties.getBaiduLocalEnabled()) {
            return BaiduApiHelper.faceDetectUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceDetectByBaidu(ImageParm image, boolean localEnabled) {
        Map<String, String> imageData = this.buildImageParmForBaidu(image);
        imageData.put("liveness_control", "NORMAL");
        imageData.put("max_face_num", "2");

        String params = new JsonHelper().toJson(imageData);
        if (localEnabled) {
            return BaiduApiHelper.faceDetectUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceCompareByBaidu(ImageParm image1, ImageParm image2) {
        return this.faceCompareByBaidu(image1, image2, properties.getBaiduExpectFaceCompareScore());
    }

    @Override
    public FaceResult faceCompareByBaidu(ImageParm image1, ImageParm image2, boolean localEnabled) {
        return this.faceCompareByBaidu(image1, image2, properties.getBaiduExpectFaceCompareScore(), localEnabled);
    }

    @Override
    public FaceResult faceCompareByBaidu(ImageParm image1, ImageParm image2, Double expectFaceCompareScore) {
        List<Map<String, String>> images = new ArrayList<>();
        images.add(this.buildImageParmForBaidu(image1));
        images.add(this.buildImageParmForBaidu(image2));

        if (expectFaceCompareScore == null) {
            expectFaceCompareScore = properties.getBaiduExpectFaceCompareScore();
        }

        String params = new JsonHelper().toJson(images);
        if (properties.getBaiduLocalEnabled()) {
            return BaiduApiHelper.faceCompareUseLocalApi(properties, params, expectFaceCompareScore);
        }
        return BaiduApiHelper.faceCompare(properties, params, expectFaceCompareScore);
    }

    @Override
    public FaceResult faceCompareByBaidu(ImageParm image1, ImageParm image2, Double expectFaceCompareScore,
                                         boolean localEnabled) {
        List<Map<String, String>> images = new ArrayList<>();
        images.add(this.buildImageParmForBaidu(image1));
        images.add(this.buildImageParmForBaidu(image2));

        if (expectFaceCompareScore == null) {
            expectFaceCompareScore = properties.getBaiduExpectFaceCompareScore();
        }

        String params = new JsonHelper().toJson(images);
        if (localEnabled) {
            return BaiduApiHelper.faceCompareUseLocalApi(properties, params, expectFaceCompareScore);
        }
        return BaiduApiHelper.faceCompare(properties, params, expectFaceCompareScore);
    }

    private Map<String, String> buildImageParmForBaidu(ImageParm image) {
        if (image == null || StringUtils.isBlank(image.value())) {
            throw new IllegalArgumentException("ImageParm must be not empty.");
        }

        // image_type: URL、BASE64、FACE_TOKEN，其中“本地化部署API”不支持 URL 方式
        Map<String, String> imageData = new HashMap<>();
        if (ImageParmType.FILE_URL == image.type()) {
            if (properties.getBaiduLocalEnabled()) {
                String imageBase64 = CommonUtils.toBase64(image.value());
                imageData.put("image", imageBase64);
                imageData.put("image_type", "BASE64");
            } else {
                imageData.put("image_type", "URL");
                imageData.put("image", image.value());
            }
        } else if (ImageParmType.FACE_TOKEN == image.type()) {
            imageData.put("image_type", "FACE_TOKEN");
            imageData.put("image", image.value());
        } else {
            imageData.put("image_type", "BASE64");
            if (ImageParmType.FILE_PATH == image.type()) {
                File imageFile = new File(image.value());
                String imageBase64 = CommonUtils.toBase64(imageFile);
                imageData.put("image", imageBase64);
            } else {
                imageData.put("image", image.value());
            }
        }
        return imageData;
    }

    private void buildImageParmForFacePlus(Map<String, String> params, ImageParm image, String index) {
        if (image == null || StringUtils.isBlank(image.value())) {
            throw new IllegalArgumentException("ImageParm must be not empty.");
        }

        // 参数：image_url、image_file、image_base64、face_token
        if (ImageParmType.FILE_URL == image.type()) {
            params.put("image_url" + (index != null ? index : ""), image.value());
        } else if (ImageParmType.FACE_TOKEN == image.type()) {
            params.put("face_token" + (index != null ? index : ""), image.value());
        } else {
            if (ImageParmType.FILE_PATH == image.type()) {
                File imageFile = new File(image.value());
                String imageBase64 = CommonUtils.toBase64(imageFile);
                params.put("image_base64" + (index != null ? "_" + index : ""), imageBase64);
            } else {
                params.put("image_base64" + (index != null ? "_" + index : ""), image.value());
            }
        }
    }

    @Override
    public FaceResult faceDetectByFacePlus(ImageParm image) {
        Map<String, String> params = new HashMap<>();
        this.buildImageParmForFacePlus(params, image, null);

        return FacePlusApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceCompareByFacePlus(ImageParm image1, ImageParm image2) {
        Map<String, String> params = new HashMap<>();
        this.buildImageParmForFacePlus(params, image1, "1");
        this.buildImageParmForFacePlus(params, image2, "2");

        return FacePlusApiHelper.faceCompare(properties, params);
    }

    public void setProperties(FaceVerifyProperties properties) {
        this.properties = properties;
    }

}
